/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_APPEARANCE_H
#define X3DTK_GL_APPEARANCE_H

#include "GL_X3DAppearanceNode.h"

namespace X3DTK {
namespace GL {

class X3DMaterialNode;
class X3DTextureNode;
class X3DTextureTransformNode;
class LineProperties;
class FillProperties;

/*! \brief Class declared in <b><X3DTK/GL/scenegraph.h></b> providing the GL implementation
 *  of X3D::Appearance.
 *
 *  \ingroup GLSceneGraph
 */
 
class Appearance : public X3DAppearanceNode
{
public:
  /// Constructor.
  Appearance();
  /// Clones the node.
  virtual SFAbstractNode clone() const;
  /// Destructor.
  virtual ~Appearance();
  
  /// Sets the Material attribute.
  void setMaterial(X3DMaterialNode *material);
  /// Sets the Texture attribute.
  void setTexture(X3DTextureNode *texture);
  /// Sets the TextureTransform attribute.
  void setTextureTransform(X3DTextureTransformNode *textureTransform);
  /// Sets the LineProperties attribute.
  void setLineProperties(LineProperties *lineProperties);
  /// Sets the FillProperties attribute.
  void setFillProperties(FillProperties *fillProperties);
  
  /// Gets the Material attribute.
  inline SFNode getMaterial() {return _material;};
  /// Gets the Texture attribute.
  inline SFNode getTexture() {return _texture;};
  /// Gets the TextureTransform attribute.
  inline SFNode getTextureTransform() {return _textureTransform;};
  /// Gets the LineProperties attribute.
  inline SFNode getLineProperties() {return _lineProperties;};
  /// Gets the FillProperties attribute.
  inline SFNode getFillProperties() {return _fillProperties;};
  
  /// Gets the children list.
  virtual MFAbstractNode getChildList() const;
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N); 

protected:  
  /// Copy constructor.	     
  Appearance(const Appearance &A);

private:  
  /// Material node.
  SFNode _material;
  /// Texture node.
  SFNode _texture;
  /// Texture transform node.
  SFNode _textureTransform;
  /// Line properties node.
  SFNode _lineProperties;
  /// FIll properties node.
  SFNode _fillProperties;
};

}
}

#endif
