/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_X3DSHAPENODE_H
#define X3DTK_GL_X3DSHAPENODE_H

#include "GL_X3DChildNode.h"

namespace X3DTK {
namespace GL {

class X3DGeometryNode;
class X3DAppearanceNode;

/*! \brief Base class declared in <b><X3DTK/GL/scenegraph.h></b> providing the GL implementation
 *  of X3D::X3DShapeNode.
 *
 *  \ingroup GLSceneGraph
 */
 class X3DShapeNode : public X3DChildNode
{
public:
  /// Constructor.
  X3DShapeNode();
  /// Virtual destructor.
  virtual ~X3DShapeNode();
  
  /// Sets the Geometry attribute.
  void setGeometry(X3DGeometryNode *geometry);
  /// Sets the Appearance attribute.
  void setAppearance(X3DAppearanceNode *appearance);

  /// Gets the Geometry attribute.
  inline SFNode getGeometry() const {return _geometry;};
  /// Gets the Appearance attribute.
  inline SFNode getAppearance() const {return _appearance;};
  
  /// Gets the children list.
  virtual MFAbstractNode getChildList() const;
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N);
  
protected:
  /// Copy constructor.	       
  X3DShapeNode(const X3DShapeNode &N);

private:  
  /// Geometry node.
  SFNode _geometry;
  /// Appearance node.
  SFNode _appearance;  
};

}
}

#endif
