namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
TemplateEdgesDrawer<MData, VData, EData, FData, RW>::TemplateEdgesDrawer() 
: X3DOnePassProcessor()
{
  stateVariables = Singleton<TemplateEdgesDrawerStateVariables<MData, VData, EData, FData, RW> >::getInstance();

  graphTraversal = new DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new TemplateEdgesDrawerCoreVisitor<MData, VData, EData, FData, RW>());
}

template<class MData, class VData, class EData, class FData, bool RW> 
TemplateEdgesDrawer<MData, VData, EData, FData, RW>::~TemplateEdgesDrawer()
{
  Singleton<TemplateEdgesDrawerStateVariables<MData, VData, EData, FData, RW> >::removeInstance();
  delete graphTraversal;
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateEdgesDrawer<MData, VData, EData, FData, RW>::draw(X3DNode *N, bool selection)
{
  #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
  if (VData::template find<MESH::VertexPointData>())
  #endif
  {
    stateVariables->setSelectionMode(selection);
    stateVariables->init();
    graphTraversal->traverse(N);
    stateVariables->finish(); 
  }  
}

template<class MData, class VData, class EData, class FData, bool RW>
TemplateVertex<MData, VData, EData, FData, RW> *TemplateEdgesDrawer<MData, VData, EData, FData, RW>::getVertex(int i) const 
{
  return stateVariables->getVertex(i);
}

template<class MData, class VData, class EData, class FData, bool RW>
const SFMatrix34f &TemplateEdgesDrawer<MData, VData, EData, FData, RW>::getMatrix(int i) const 
{
  return stateVariables->getMatrix(i);
}

}
}  
