namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
TemplateFacesDrawer<MData, VData, EData, FData, RW>::TemplateFacesDrawer()
: X3DOnePassProcessor()
{
  stateVariables = Singleton<TemplateFacesDrawerStateVariables<MData, VData, EData, FData, RW> >::getInstance();

  graphTraversal = new DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new TemplateFacesDrawerCoreVisitor<MData, VData, EData, FData, RW>());
}

template<class MData, class VData, class EData, class FData, bool RW>
TemplateFacesDrawer<MData, VData, EData, FData, RW>::~TemplateFacesDrawer()
{
  Singleton<TemplateFacesDrawerStateVariables<MData, VData, EData, FData, RW> >::removeInstance();
  delete graphTraversal;
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateFacesDrawer<MData, VData, EData, FData, RW>::changeModel()
{
  stateVariables->clearGLNodes();
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateFacesDrawer<MData, VData, EData, FData, RW>::draw(X3DNode *N, bool selection)
{
  #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
  if (VData::template find<MESH::VertexPointData>())
  #endif
  {
    stateVariables->setSelectionMode(selection);
    stateVariables->init();
    graphTraversal->traverse(N);
    stateVariables->finish(); 
  }  
}

template<class MData, class VData, class EData, class FData, bool RW>
TemplateMesh<MData, VData, EData, FData, RW> *TemplateFacesDrawer<MData, VData, EData, FData, RW>::getMesh(int i) const 
{
  return stateVariables->getMesh(i);
}

template<class MData, class VData, class EData, class FData, bool RW>
const SFMatrix34f &TemplateFacesDrawer<MData, VData, EData, FData, RW>::getMatrix(int i) const 
{
  return stateVariables->getMatrix(i);
}

template<class MData, class VData, class EData, class FData, bool RW>
SFTemplateFace<MData, VData, EData, FData, RW> *TemplateFacesDrawer<MData, VData, EData, FData, RW>::getFaceOfMesh(int i, int j) const 
{
  return (stateVariables->getMesh(j)->getFaces())[i];
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateFacesDrawer<MData, VData, EData, FData, RW>::setRenderingMode(FaceMode mode)
{
  stateVariables->setRenderingMode(mode);
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateFacesDrawer<MData, VData, EData, FData, RW>::setSelectedNodes(const MFNode &selectedNodes)
{
  stateVariables->setSelectedNodes(selectedNodes);
}

}
}  
