namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
TemplateNormalsUpdater<MData, VData, EData, FData, RW>::TemplateNormalsUpdater()
: X3DMultiPassProcessor()
{
  stateVariables = Singleton<TemplateNormalsUpdaterStateVariables<MData, VData, EData, FData, RW> >::getInstance();
  
  graphTraversalMap["Faces"] = new DFSGraphTraversal();
  graphTraversalMap["Vertices"] = new DFSGraphTraversal();

  graphTraversalMap["Faces"]->setComponentVisitor(new TemplateNormalsUpdaterFacesCoreVisitor<MData, VData, EData, FData, RW>());
  graphTraversalMap["Vertices"]->setComponentVisitor(new TemplateNormalsUpdaterVerticesCoreVisitor<MData, VData, EData, FData, RW>());
}

template<class MData, class VData, class EData, class FData, bool RW>
TemplateNormalsUpdater<MData, VData, EData, FData, RW>::~TemplateNormalsUpdater()
{
  for (MFGraphTraversal::iterator it = graphTraversalMap.begin(); it != graphTraversalMap.end(); ++it)
    delete (*it).second; 
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateNormalsUpdater<MData, VData, EData, FData, RW>::updateNormals(Scene *S)
{
  updateNormals(static_cast<X3DNode *>(S));
  
  S->getProcessHistory().addEntry("MESH::TemplateNormalsUpdater");
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateNormalsUpdater<MData, VData, EData, FData, RW>::updateNormals(X3DNode *N)
{
  #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
  if (VData::template find<MESH::VertexNormalData>() 
   && FData::template find<MESH::FaceNormalData>())
  #endif 
  {
    graphTraversalMap["Faces"]->traverse(N);
  }
  #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
  if (VData::template find<MESH::VertexNormalData>() 
   && FData::template find<MESH::FaceNormalData>())
  #endif 
  { 
    graphTraversalMap["Vertices"]->traverse(N);
  }  
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateNormalsUpdater<MData, VData, EData, FData, RW>::setSelectedNodes(const MFNode &selectedNodes)
{
  stateVariables->setSelectedNodes(selectedNodes);
}

}
}
