/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_SHAPE_H
#define X3DTK_MESH_SHAPE_H

#include "MESH_SceneGraphTypes.h"
#include "MESH_X3DNode.h"
#include "MESH_X3DMeshNode.h"

namespace X3DTK {
namespace X3D {

class X3DAppearanceNode;

}

namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/scenegraph.h></b> defining a shape
 *  for the MESH scene graph.
 *
 *  \ingroup MeshSceneGraph
 */

class Shape : public X3DNode
{
public:
  /// Constructor.
  Shape();
  /// Constructor.
  Shape(X3D::X3DAppearanceNode *appearance, X3DMeshNode *mesh);
  /// Clones the node.
  virtual SFAbstractNode clone() const;
  /// Virtual destructor.
  virtual ~Shape();
  
  /// Sets the Mesh node.
  void setMesh(X3DMeshNode *mesh);
  /// Sets the X3DAppearanceNode.
  void setAppearance(X3D::X3DAppearanceNode *N);
  
  /// Gets the Mesh node.
  inline SFNode getMesh() const {return _mesh;};
  /// Gets the X3DAppearanceNode.
  inline X3D::SFNode getAppearance() const {return _appearance;};

  /// Gets the children list.
  virtual MFAbstractNode getChildList() const;
  
   /// Adds a child to the Children attribute. Returns TRUE if succeeds.
  bool addChild(const SFAbstractNode &N);
  /// Sets the child. Behaves like addChild.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child. Return TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);

protected:
  /// Copy constructor.
  Shape(const Shape &S);

private:
  X3D::SFNode _appearance;
  SFNode _mesh;
};

}
}

#endif
