namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
TemplateStatsCalculator<MData, VData, EData, FData, RW>::TemplateStatsCalculator()
: X3DOnePassProcessor()
{
  stateVariables = Singleton<TemplateStatsCalculatorStateVariables<MData, VData, EData, FData, RW> >::getInstance();

  graphTraversal = new DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new TemplateStatsCalculatorCoreVisitor<MData, VData, EData, FData, RW>());
}

template<class MData, class VData, class EData, class FData, bool RW>
TemplateStatsCalculator<MData, VData, EData, FData, RW>::~TemplateStatsCalculator()
{
  Singleton<TemplateStatsCalculatorStateVariables<MData, VData, EData, FData, RW> >::removeInstance();
  delete graphTraversal;
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateStatsCalculator<MData, VData, EData, FData, RW>::calculate(X3DNode *N)
{
  stateVariables->init();
  graphTraversal->traverse(N);
  stateVariables->finish(); 
}

template<class MData, class VData, class EData, class FData, bool RW>
void TemplateStatsCalculator<MData, VData, EData, FData, RW>::setSelectedNodes(const MFNode &selectedNodes)
{
  stateVariables->setSelectedNodes(selectedNodes);
}

template<class MData, class VData, class EData, class FData, bool RW>
MFNode TemplateStatsCalculator<MData, VData, EData, FData, RW>::getSelectedNodes() const
{
  return stateVariables->getSelectedNodes();
}

template<class MData, class VData, class EData, class FData, bool RW>
unsigned int TemplateStatsCalculator<MData, VData, EData, FData, RW>::getModelFaces() const
{
  return stateVariables->getModelFaces();
}

template<class MData, class VData, class EData, class FData, bool RW>
unsigned int TemplateStatsCalculator<MData, VData, EData, FData, RW>::getModelVertices() const
{
  return stateVariables->getModelVertices();
}

template<class MData, class VData, class EData, class FData, bool RW>
unsigned int TemplateStatsCalculator<MData, VData, EData, FData, RW>::getModelEdges() const
{
  return stateVariables->getModelEdges();
}

template<class MData, class VData, class EData, class FData, bool RW>
unsigned int TemplateStatsCalculator<MData, VData, EData, FData, RW>::getFacesOf(X3DNode *N) const
{
  return stateVariables->getFacesOf(N);
}

template<class MData, class VData, class EData, class FData, bool RW>
unsigned int TemplateStatsCalculator<MData, VData, EData, FData, RW>::getVerticesOf(X3DNode *N) const
{
  return stateVariables->getVerticesOf(N);
}

template<class MData, class VData, class EData, class FData, bool RW>
unsigned int TemplateStatsCalculator<MData, VData, EData, FData, RW>::getEdgesOf(X3DNode *N) const
{
  return stateVariables->getEdgesOf(N);
}

template<class MData, class VData, class EData, class FData, bool RW>
const SFMatrix34f &TemplateStatsCalculator<MData, VData, EData, FData, RW>::getMatrixOf(X3DNode *N)
{
  return stateVariables->getMatrixOf(N);
}

template<class MData, class VData, class EData, class FData, bool RW>
const TemplateMeshInfo<MData, VData, EData, FData, RW> &TemplateStatsCalculator<MData, VData, EData, FData, RW>::getModelMeshInfo() const
{
  return stateVariables->getModelMeshInfo();
}

template<class MData, class VData, class EData, class FData, bool RW>
const std::list<std::pair<X3DNode *, TemplateMeshInfo<MData, VData, EData, FData, RW> > > &TemplateStatsCalculator<MData, VData, EData, FData, RW>::getMeshInfoList() const
{
  return stateVariables->getMeshInfoList();
}

template<class MData, class VData, class EData, class FData, bool RW>
const TemplateVertexInfo<MData, VData, EData, FData, RW> &TemplateStatsCalculator<MData, VData, EData, FData, RW>::getModelVertexInfo() const
{
  return stateVariables->getModelVertexInfo();
}

template<class MData, class VData, class EData, class FData, bool RW>
const std::list<std::pair<X3DNode *, TemplateVertexInfo<MData, VData, EData, FData, RW> > > &TemplateStatsCalculator<MData, VData, EData, FData, RW>::getVertexInfoList() const
{
  return stateVariables->getVertexInfoList();
}


}
}  
