/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_INLINE_H
#define X3DTK_X3D_INLINE_H

#include "X3D_X3DChildNode.h"
#include "X3D_X3DUrlObject.h"
#include "X3D_X3DBoundedObject.h"

namespace X3DTK {
namespace X3D {

/*! \brief Base class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/networking.html">Networking 
 *  component</a> defining an Inline.
 *  (<a href="../../doc/X3DSpecs/Part01/components/networking.html#Inline">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class Inline : public X3DChildNode, public X3DUrlObject, public X3DBoundedObject
{
public:
  /// Constructor.
  Inline();
  /// Constructor.
  Inline(const SFBool &load);
  /// Clone method.
  virtual SFAbstractNode clone() const;
  
  /// Virtual destructor.
  virtual ~Inline();
  
  /// Sets the url attribute.
  void setLoad(const SFBool &load);

  /// Gets the url attribute.
  inline const SFBool &getLoad() const {return _load;};

  /// Load method.
  virtual void load(const X3DFileElement *element);
  /// Write method.
  virtual SFString &write(SFString &output) const;

protected:
  /// Copy constructor.
  Inline(const Inline &I);  

private:  
  /// url.
  SFBool _load;  
};

}
}

#endif
