////////////////////////////////////////////////////////////////////////////////
//                            X3D_MeshBuilder.inl                             //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace X3D {

template<class MData, class VData, class EData, class FData, bool RW>
TemplateMeshBuilder<MData, VData, EData, FData, RW>::TemplateMeshBuilder() 
{
  stateVariables = Singleton<TemplateMeshBuilderStateVariables<MData, VData, EData, FData, RW> >::getInstance();
  graphTraversal = new DFSGraphTraversal();

  graphTraversal->setComponentVisitor(new TemplateMeshBuilderCoreVisitor<MData, VData, EData, FData, RW>());
  graphTraversal->setComponentVisitor(new TemplateMeshBuilderGeometry3DVisitor<MData, VData, EData, FData, RW>());
  graphTraversal->setComponentVisitor(new TemplateMeshBuilderGroupingVisitor<MData, VData, EData, FData, RW>());
  graphTraversal->setComponentVisitor(new TemplateMeshBuilderShapeVisitor<MData, VData, EData, FData, RW>());
  graphTraversal->setComponentVisitor(new TemplateMeshBuilderRenderingVisitor<MData, VData, EData, FData, RW>());
}

template<class MData, class VData, class EData, class FData, bool RW>
TemplateMeshBuilder<MData, VData, EData, FData, RW>::~TemplateMeshBuilder() 
{
  Singleton<TemplateMeshBuilderStateVariables<MData, VData, EData, FData, RW> >::removeInstance();

  delete graphTraversal;
};

template<class MData, class VData, class EData, class FData, bool RW> 
MESH::SFNode TemplateMeshBuilder<MData, VData, EData, FData, RW>::build(SFNode N)
{
  stateVariables->init();
  graphTraversal->traverse(N);
  stateVariables->finish();
  
  return stateVariables->getRoot();
}

template<class MData, class VData, class EData, class FData, bool RW>
MESH::Scene *TemplateMeshBuilder<MData, VData, EData, FData, RW>::build(Scene *S)
{
  return static_cast<MESH::Scene *>(build(static_cast<SFNode>(S)));
};
  
}
}  
