/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_SPHERE_H
#define X3DTK_X3D_SPHERE_H

#include "X3DTypes.h"
#include "X3D_X3DGeometry3DNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/geometry3D.html">
 *  Geometry3D component</a> defining a Sphere
 *  (<a href="../../doc/X3DSpecs/Part01/components/geometry3D.html#Sphere">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class Sphere : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Sphere();
  /// Constructor from its attribute.
  explicit Sphere(const SFFloat &radius);
  /// Clones the node.
  virtual SFAbstractNode clone() const;
  /// Destructor.
  virtual ~Sphere();
  
  /// Sets the radius attribute.
  void setRadius(const SFFloat &radius);
  
  /// Gets the radius attribute.
  inline const SFFloat &getRadius() const {return _radius;};
  
  /// Loads the attributes from a X3DFileElement e.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString &write(SFString &output) const;

protected:
  /// Copy constructor.
  Sphere(const Sphere &C);

private:
  ///  Radius attribute.
  SFFloat _radius;
};

}
}

#endif
