/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_TRIANGLEFANSET_H
#define X3DTK_X3D_TRIANGLEFANSET_H

#include "X3DTypes.h"
#include "X3D_X3DComposedGeometryNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
 *  Rendering component</a> defining a TriangleFanSet
 *  (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#TriangleFanSet">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class TriangleFanSet : public X3DComposedGeometryNode
{
public:
  /// Constructor.
  TriangleFanSet();
  /// Constructs a triangleFanSet from its attributes.
  TriangleFanSet(X3DColorNode *color,
                 X3DCoordinateNode *coord,
                 X3DNormalNode *normal,
                 X3DTextureCoordinateNode *texCoord,
                 const SFBool &ccw,
                 const SFBool &colorPerVertex,
                 const SFBool &convex,
                 const SFFloat &creaseAngle,
                 const MFInt32 &fanCount,
                 const SFBool &normalPerVertex,
                 const SFBool &solid);
  /// Clones the node.
  virtual SFAbstractNode clone() const;
  /// Destructor.
  virtual ~TriangleFanSet();
  
  /// Sets the Convex attribute.
  void setConvex(const SFBool &convex);
  /// Sets the CreaseAngle attribute.
  void setCreaseAngle(const SFFloat &creaseAngle);
  /// Sets the FanCount attribute.
  void setFanCount(const MFInt32 &fanCount);
  
  /// Gets the Convex attribute.
  inline const SFBool &getConvex() const {return _convex;};
  /// Gets the CreaseAngle attribute.
  inline const SFFloat &getCreaseAngle() const {return _creaseAngle;};
  /// Gets the FanCount attribute.
  inline const MFInt32 &getFanCount() const {return _fanCount;};

  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString &write(SFString &output) const;

protected:
  /// Copy constructor.         
  TriangleFanSet(const TriangleFanSet &T);

private:
  /// Convex attribute.
  SFBool _convex;  
  /// Crease angle attribute.
  SFFloat _creaseAngle;
  /// Fan count attribute.
  MFInt32 _fanCount; 
};

}
}

#endif
