#include "X3D_Info.h"
#include "X3D_Price.h"
#include "X3D_Modeler.h"

namespace X3DTK {
namespace X3D {

Info::Info()
: X3DChildNode(), _date("0/0/0"), _price(0), _modeler(0)
{
  // Defines the tag of the node. This string is the one read in the X3D file.
  defineTypeName("Info");
}

Info::Info(const SFString &date, SFNode price, SFNode modeler)
: X3DChildNode(), _date(date), _price(price), _modeler(modeler)
{
  // Defines the tag of the node.
  defineTypeName("Info");
}

Info::Info(const Info &I)
: X3DChildNode(I), _date(I._date), _price(I._price), _modeler(I._modeler)
{
}

Info::~Info()
{
  removeParentFromChild(this, _price);
  removeParentFromChild(this, _modeler);
}

SFAbstractNode Info::clone() const
{
  return new Info(*this);
}

void Info::setDate(const SFString &date)
{
  _date = date;
}

void Info::setPrice(Price *price)
{
  removeParentFromChild(this, _price);
  _price = price;
  addParentToChild(this, _price);
}

void Info::setModeler(Modeler *modeler)
{
  removeParentFromChild(this, _modeler);
  _modeler = modeler;
  addParentToChild(this, _modeler);
}

MFAbstractNode Info::getChildList() const
{
  MFAbstractNode childrenList;
  if (_price != 0)
    childrenList.push_back(_price);
  if (_modeler != 0)
    childrenList.push_back(_modeler);  
    
  return childrenList; 
}

bool Info::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Price *>(N) != 0)
  {
    if (_price != 0)
      return false;
      
    setPrice(static_cast<Price *>(N));
    return true;
  }
  
  if (dynamic_cast<Modeler *>(N) != 0)
  {
    if (_modeler != 0)
      return false;
      
    setModeler(static_cast<Modeler *>(N));
    return true;    
  }
  
  return false;
}

bool Info::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Price *>(N) != 0)
  {
    setPrice(static_cast<Price *>(N));
    return true;
  }
  
  if (dynamic_cast<Modeler *>(N) != 0)
  {
    setModeler(static_cast<Modeler *>(N));
    return true;    
  }
  
  return false;
}

bool Info::removeChild(const SFAbstractNode &N)
{
  if (_price == N)
  {
    setPrice(0);
    return true;
  }
  
  if (_modeler == N)
  {
    setModeler(0);
    return true;
  }
  
  return false;
}

void Info::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("date");
  if (index != -1)
    _date = element->getAttribute(index);
}

SFString Info::write() const
{
  SFString attr;
  if (_date != "0/0/0")    
    attr += " date=\"" + toSFString(_date) + "\"";
      
  return attr;
}

}
}
