#include "X3D_FCylinder.h"

namespace X3DTK {
namespace X3D {

FCylinder::FCylinder()
: Cylinder(), _section(8)
{
  // Defines the tag of the node. This string is the one read in the X3D file.
  defineTypeName("FCylinder");
}

FCylinder::FCylinder(SFBool bottom, SFFloat radius, SFFloat height, SFBool side, SFBool top, SFInt32 section)
: Cylinder(bottom, radius, height, side, top), _section(section)
{
  // Defines the tag of the node.
  defineTypeName("FCylinder");
}

FCylinder::FCylinder(const FCylinder &C)
: Cylinder(C), _section(C._section)
{
}

SFAbstractNode FCylinder::clone() const
{
  return new FCylinder(*this);
}

void FCylinder::setSection(SFInt32 section)
{
  _section = section;
}

void FCylinder::load(const X3DFileElement *element)
{
  Cylinder::load(element);
  
  int index;
  index = element->getIndexAttribute("section");
  if (index != -1)
    _section = element->getAttribute(index).toInt();
}

SFString &FCylinder::write(SFString &output) const
{
  Cylinder::write(output);
  if (_section != 8)    
    output += " section=\"" + toSFString(_section) + "\"";
      
  return output;
}

}
}
