#include "X3D_InfoShape.h"
#include "X3D_Price.h"
#include "X3D_Modeler.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

InfoShape::InfoShape()
: Shape(), _date("0/0/0"), _price(0), _modeler(0)
{
  // Defines the tag of the node. This string is the one read in the X3D file.
  // We give the name "Shape" to erase the older definition.
  defineTypeName("Shape", "MyComponent");
}

InfoShape::InfoShape(const InfoShape &I)
: Shape(I), _date(I._date), _price(I._price), _modeler(I._modeler)
{
}

InfoShape::~InfoShape()
{
  removeParentFromChild(this, _price);
  removeParentFromChild(this, _modeler);
}

SFAbstractNode InfoShape::clone() const
{
  return new InfoShape(*this);
}

void InfoShape::setDate(const SFString &date)
{
  _date = date;
}

void InfoShape::setPrice(Price *price)
{
  removeParentFromChild(this, _price);
  _price = price;
  addParentToChild(this, _price);
}

void InfoShape::setModeler(Modeler *modeler)
{
  removeParentFromChild(this, _modeler);
  _modeler = modeler;
  addParentToChild(this, _modeler);
}

MFAbstractNode InfoShape::getChildList() const
{
  MFAbstractNode childrenList = Shape::getChildList();
  if (_price != 0)
    childrenList.push_back(_price);
  if (_modeler != 0)
    childrenList.push_back(_modeler);  
    
  return childrenList; 
}

bool InfoShape::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Price *>(N) != 0)
  {
    if (_price != 0)
      return false;
      
    setPrice(static_cast<Price *>(N));
    return true;
  }
  
  if (dynamic_cast<Modeler *>(N) != 0)
  {
    if (_modeler != 0)
      return false;
      
    setModeler(static_cast<Modeler *>(N));
    return true;    
  }
  
  Shape::addChild(N);
  
  return false;
}

bool InfoShape::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Price *>(N) != 0)
  {
    setPrice(static_cast<Price *>(N));
    return true;
  }
  
  if (dynamic_cast<Modeler *>(N) != 0)
  {
    setModeler(static_cast<Modeler *>(N));
    return true;    
  }
  
  Shape::setChild(N);
  
  return false;
}

bool InfoShape::removeChild(const SFAbstractNode &N)
{
  if (_price == N)
  {
    setPrice(0);
    return true;
  }
  
  if (_modeler == N)
  {
    setModeler(0);
    return true;
  }
  
  Shape::removeChild(N);
  
  return false;
}

void InfoShape::load(const X3DFileElement *element)
{
  Shape::load(element);

  int index;
  index = element->getIndexAttribute("date");
  if (index != -1)
    _date = element->getAttribute(index);
}

}
}
